/*
* Copyright 2009(c) Theo Pavlidis
* See PixWin.h for a detailed notice on the use of the software
*/	

// PixWindow.cpp : implementation file
//

#include "stdafx.h"
#include "BlockComp.h"
#include "PixWindow.h"
#include <mmsystem.h>	// for timers

// CPixWindow

IMPLEMENT_DYNAMIC(CPixWindow, CWnd)

CPixWindow::CPixWindow()
{
	CString strWndClass = AfxRegisterWndClass (
		0,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW),
		(HBRUSH)::GetStockObject(WHITE_BRUSH),
		AfxGetApp()->LoadStandardIcon(IDI_WINLOGO)
	);

	CreateEx(WS_EX_OVERLAPPEDWINDOW, strWndClass, _T("PixWindow"),
		WS_POPUPWINDOW | WS_THICKFRAME | WS_CAPTION | WS_MAXIMIZEBOX | WS_MINIMIZEBOX, // style
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, // x, y, w, h
		NULL, // parent
		NULL, // menu
		NULL);

	m_Mat.mat = NULL;
}

CPixWindow::~CPixWindow()
{
	if(m_Mat.mat != NULL) delete[] m_Mat.mat;
}

int CPixWindow::PopWindow(int x0, int y0, int k, int nC, int bxsz, BOOL useIcons)
{
	CPaintDC dc(this);	// to get properties of screen
	int ScreenWidth  = dc.GetDeviceCaps(HORZRES);
	int ScreenHeight = dc.GetDeviceCaps(VERTRES);

	m_UseIcons = useIcons;
	if(useIcons==TRUE) {
		m_IconList[0] = ::AfxGetApp()->LoadIconA(IDI_ICON_F1);
		m_IconList[1] = ::AfxGetApp()->LoadIconA(IDI_ICON_F2);
		m_IconList[2] = ::AfxGetApp()->LoadIconA(IDI_ICON_F3);
		m_IconList[3] = ::AfxGetApp()->LoadIconA(IDI_ICON_F5);
		bxsz = 60;	// temporary cludge - assumes icons size is 32x32
	}
	int width  = 3*W_OFF + 2*k*bxsz;
	int height = 2*H_OFF + k*bxsz;

	if(width > ScreenWidth || height > ScreenHeight) {
		::AfxMessageBox("Too big an Image");
		return -1;
	}

	CString name;
	name.Format("Test for %d by %d", k, k);
	SetWindowText(name);

	m_Mat.K = k;
	m_Mat.nC = nC;
	m_Mat.boxsz = bxsz;

	if(m_Mat.fill() == FALSE) {
		::AfxMessageBox("Cannot allocate matrix space");
		return -1;
	}

	// +12 and +40 are estimates of frame size
	SetWindowPos(&wndTop, x0, y0, width+12, height+40, SWP_SHOWWINDOW);
	ShowWindow(SW_RESTORE);
	Invalidate(FALSE);
	timeBeginPeriod(1);
	m_start_time = timeGetTime();
	m_msg = "";

	return 1;
}


BEGIN_MESSAGE_MAP(CPixWindow, CWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
END_MESSAGE_MAP()



// CPixWindow message handlers




namespace {
	void DrawMatrix(CDC *dcp, matpair *mp, int xoff, int yoff)
	{
		int i, x, y, x0, y0;
		CPen *oldPen = (CPen *)dcp->SelectStockObject(NULL_PEN);
		CBrush RedBrush(RGB(255, 0, 0));
		CBrush GreenBrush(RGB(0, 255, 0));
		CBrush BlueBrush(RGB(0, 0, 255));
		CBrush QBrush(RGB(128, 128, 128));
		dcp->SelectStockObject(WHITE_BRUSH);
		dcp->Rectangle(xoff, yoff, xoff+mp->K*mp->boxsz, yoff+mp->K*mp->boxsz); 
		for(i=0, y=0; y<mp->K; y++) {
			for(x=0; x<mp->K; x++, i++) {
				switch(mp->mat[i]){
					case 0: continue;
					case 1: dcp->SelectStockObject(BLACK_BRUSH);
						break;
					case 2: dcp->SelectObject(RedBrush);
						break;
					case 3:  dcp->SelectObject(GreenBrush);
						break;
					case 4: dcp->SelectObject(BlueBrush);
						break;
					default:  dcp->SelectObject(QBrush);
				}
				x0 = xoff + x*mp->boxsz;
				y0 = yoff + y*mp->boxsz;
				dcp->Rectangle(x0, y0, x0+mp->boxsz+1, y0+mp->boxsz+1);
			}
		}
		dcp->SelectObject(oldPen);
	}

	void DrawIconMatrix(CDC *dcp, matpair *mp, int xoff, int yoff, HICON icon[])
	{
		int i, x, y, x0, y0;
		CBrush SeaBrush(RGB(0, 200, 255));
		dcp->SelectObject(SeaBrush);
		dcp->Rectangle(xoff, yoff, xoff+mp->K*mp->boxsz, yoff+mp->K*mp->boxsz); 
		for(i=0, y=0; y<mp->K; y++) {
			for(x=0; x<mp->K; x++, i++) {
				x0 = xoff + x*mp->boxsz + 14;	// 14 is a cludge
				y0 = yoff + y*mp->boxsz + 14;	// 14 is a cludge
				if(mp->mat[i]>0) {
					int ki = (mp->mat[i]-1)%4;		// only four icons for now
					dcp->DrawIcon(x0, y0, icon[ki]);
				}
			}
		}
	}
}

void CPixWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	CRect r;
	GetClientRect(&r);
	dc.SelectStockObject(GRAY_BRUSH);
	dc.Rectangle(&r);

	int span = 2*W_OFF+m_Mat.K*m_Mat.boxsz;
	if(m_UseIcons==TRUE) DrawIconMatrix(&dc, &m_Mat, W_OFF, H_OFF, m_IconList);
	else DrawMatrix(&dc, &m_Mat, W_OFF, H_OFF);
	int vtmp = m_Mat.mat[m_Mat.flip];
	m_Mat.mat[m_Mat.flip] = m_Mat.nvalue;
	if(m_UseIcons==TRUE) DrawIconMatrix(&dc, &m_Mat, span, H_OFF, m_IconList);
	else DrawMatrix(&dc, &m_Mat, span, H_OFF);
	m_Mat.mat[m_Mat.flip] = vtmp;
	if(m_msg.GetLength()>1) {	// we found the difference
		dc.SelectStockObject(NULL_BRUSH);
		dc.TextOutA(m_point.x-20, 5, m_msg);	// offsets determined by trial and error
		int d = m_Mat.boxsz/3;	// to make the circle a bit smaller than the block
		dc.Ellipse(m_point.x-d, m_point.y-d, m_point.x+d, m_point.y+d);
		if(m_point.x > span) m_point.x -= (span-W_OFF);
		else m_point.x += (span-W_OFF);
		dc.Ellipse(m_point.x-d, m_point.y-d, m_point.x+d, m_point.y+d);
	}
}

void CPixWindow::OnLButtonDown(UINT nFlags, CPoint point)
{
	// Find Grid
	int x = point.x - W_OFF;
	int y = point.y - H_OFF;
	if(x<0 || y<0) goto failure;
	int mat_width = m_Mat.boxsz*m_Mat.K;
	if(y > mat_width) goto failure;
	y /= m_Mat.boxsz;
	if(x<mat_width) {	// we are on the left
		x /= m_Mat.boxsz;
	}
	else { // we may be on the right
		x -= (W_OFF + mat_width);
		if(x<0 || x>mat_width) goto failure;
		x /= m_Mat.boxsz;
	}
	if(y*m_Mat.K+x == m_Mat.flip) {
		int t = timeGetTime();
		double tt = (t-m_start_time)/1000;
		timeEndPeriod(1);
		m_msg.Format("Time %5.1f secs", tt);
		m_point = point;
		Invalidate(FALSE);
		::MessageBeep(0x3333);
		m_stat.total += tt;
		m_stat.n++;
		return;
	}
failure:
	::MessageBeep(1);
	return;
}

void CPixWindow::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_msg = "";
	Invalidate(FALSE);
}
